<?php
  require_once "redirect.php";

  function passthroughRedirect($filename) {
	if (isset($_SERVER['HTTP_IF_MODIFIED_SINCE'])) {
	        $if_modified_since = preg_replace('/;.*$/', '', $_SERVER['HTTP_IF_MODIFIED_SINCE']);
	}
	$mtime = filemtime($filename);
	if (isset($if_modified_since)) {
		$time_since = strtotime($if_modified_since);
		if (isset($time_since) && $time_since !== -1) {
			if ($time_since >= $mtime) {
				header("HTTP/1.0 304 Not Modified");
				exit;
			}
		}
	}

	header('Status: 200 OK');
	header('Cache-Control: public');
	header('Expires: ' . dateRfc1234(mktime() + 60*60*24*7));
	header('Last-Modified: ' . dateRfc1234($mtime));	
	header('Content-Transfer-Encoding: binary');
	header('Content-Length: ' . filesize($filename));
	header('Content-Type: ' . mime($filename));
	header('File-Name: ' . $filename);
	readfile($filename);
	exit;
  }

  function mime($filename) {
    //$finfo = new finfo(FILEINFO_MIME, "/usr/share/misc/file/magic.mime");
    //if (!finfo) {
    //  return null;
    //}
    //return $finfo->file($filename);

    $suffix  = strrchr($filename, ".");
    switch ($suffix) {
    case '.jpg':
    case '.jpeg':
      return 'image/jpeg';
    case '.gif':
      return 'image/gif';
    case '.png':
      return 'image/png';
    }
    //return null;
    return 'image';
  }

//RFC 2616: rfc1123-date
  function dateRfc1234($filetime) {
    return gmdate('D, d M Y H:i:s', $filetime) . ' GMT';
  }

  function doConvert($img, $hue, $sat, $value, $imgOutput) {
    $img = escapeshellarg($img);
    $modulate = escapeshellarg("$value,$sat,$hue");
    $imgOutputEsc = escapeshellarg($imgOutput);
    $retval = null;

    //a bug in imagemagick 6.7 forces us to convert to truecolor first
    $cmd = "convert $img -type truecolor -modulate $modulate $imgOutputEsc";
    exec($cmd, $output, $retval);

    if ($retval == 0) {
      passthroughRedirect($imgOutput);
    } else {
      redirect("error.gif", 302);
    }
  }

  $hue = 100;
  $sat = 100;
  $value = 100;
  $img = $_REQUEST['img'];

  if (isset($_REQUEST['hue'])) {
    $hue = intval($_REQUEST['hue']);
  }

  if (isset($_REQUEST['sat'])) {
    $sat = intval($_REQUEST['sat']);
  }

  if (isset($_REQUEST['value'])) {
    $value = intval($_REQUEST['value']);
  }

  $hue = min(300, max(0, $hue));
  $sat = min(1000, max(0, $sat));
  $value = min(1000, max(0, $value));

  if ($img && !preg_match("/\.\./", $img)) {
    $imgOutput = "hsv/{$hue}_{$sat}_{$value}_{$img}";

    if (is_file($img)) {
      if (!is_file($imgOutput)) {
        doConvert($img, $hue, $sat, $value, $imgOutput);
      } else {
        $lastChange = max(filemtime($img), filemtime($_SERVER['SCRIPT_FILENAME']));
        $destChange = filemtime($imgOutput);
        if ($destChange < $lastChange) {
          doConvert($img, $hue, $sat, $value, $imgOutput);
        } else {
          passthroughRedirect($imgOutput);
        }
      }
//      if ($retval == 0) {
//        passthroughRedirect($imgOutput);
//      }
    }
  }
  redirect("error.gif", 302);

?>