package com.analogbit.puzzletools.client;

import com.analogbit.puzzletools.client.slidingblocks.gui.ErrorDialogBox;
import com.analogbit.puzzletools.client.slidingblocks.gui.InitialSetupPanel;
import com.analogbit.puzzletools.client.slidingblocks.gui.KeyPanel;
import com.analogbit.puzzletools.client.slidingblocks.gui.PhaseListener;
import com.analogbit.puzzletools.client.slidingblocks.gui.SolutionPanel;
import com.analogbit.puzzletools.client.slidingblocks.gui.StatusDialogBox;
import com.analogbit.puzzletools.client.slidingblocks.state.BlockPuzzleSolver;
import com.analogbit.puzzletools.client.slidingblocks.state.BlockPuzzleSolverListener;
import com.analogbit.puzzletools.client.slidingblocks.state.GoalState;
import com.google.gwt.core.client.EntryPoint;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.ui.RootPanel;
import com.google.gwt.user.client.ui.VerticalPanel;

/**
 * Entry point classes define <code>onModuleLoad()</code>.
 */
public class MainApp implements EntryPoint {
	private InitialSetupPanel initialPanel = new InitialSetupPanel();
	private KeyPanel keyPanel = new KeyPanel();
	private SolutionPanel solutionPanel = new SolutionPanel();

	/**
	 * This is the entry point method.
	 */
	public void onModuleLoad() {
		final VerticalPanel vPanel = new VerticalPanel();
		vPanel.addStyleName("widePanel");
		vPanel.setHorizontalAlignment(VerticalPanel.ALIGN_CENTER);

		initialPanel.addPhaseListener(new PhaseListener() {
			public void onPhaseNext() {
				keyPanel.setGrid(initialPanel.getGrid());
				vPanel.remove(initialPanel);
				vPanel.add(keyPanel);
			}

			public void onPhasePrev() {
			}
		});
		
		keyPanel.addPhaseListener(new PhaseListener() {
			public void onPhaseNext() {
				GoalState goal = keyPanel.getGoalState();
				
				final BlockPuzzleSolver newSolver = new BlockPuzzleSolver(initialPanel.getGrid(), goal);

				final StatusDialogBox statusBox = new StatusDialogBox();
				statusBox.addCancelClickHandler(new ClickHandler() {
					public void onClick(ClickEvent event) {
						newSolver.cancelSolving();
					}
				});
				statusBox.center();
				statusBox.show();

				newSolver.addListener(new BlockPuzzleSolverListener() {
					public void puzzleComplete(int numSteps) {
						statusBox.hide();
						
						switch (numSteps) {
						default: 
							solutionPanel.setSolution(initialPanel.getGrid(), newSolver.getAnswer());
							
							vPanel.remove(keyPanel);
							vPanel.add(solutionPanel);
							break;
						case BlockPuzzleSolver.PUZZLE_FAILED:
							ErrorDialogBox errorBox = new ErrorDialogBox("The puzzle is not solvable!");
							errorBox.center();
							errorBox.show();
							break;
						case BlockPuzzleSolver.PUZZLE_CANCELLED:
							break;
						}
					}

					public void puzzleStatusUpdate(int depth,
							int positionsPending,
							long positionsEvaluated,
							long duplicatePositions) {

						statusBox.setDepth(depth);
						statusBox.setPending(positionsPending);
						statusBox.setPositionsEvaluated(positionsEvaluated);
						statusBox.setDuplicatePositions(duplicatePositions);
						
					}
				});
				//DeferredCommand.addCommand(newSolver);
				newSolver.beginSolving();				

			}

			public void onPhasePrev() {
				vPanel.remove(keyPanel);
				vPanel.add(initialPanel);
			}
		});

		solutionPanel.addPhaseListener(new PhaseListener() {

			public void onPhaseNext() {
			}

			public void onPhasePrev() {
				vPanel.remove(solutionPanel);
				vPanel.add(keyPanel);
			}
			
		});
		
		vPanel.add(initialPanel);

		RootPanel.get("puzzletools").add(vPanel);
	}
	
}
