package com.analogbit.puzzletools.client.slidingblocks.gui;

import com.analogbit.puzzletools.client.util.NumericKeyboardListener;
import com.analogbit.puzzletools.client.util.OkCancelPanel;
import com.analogbit.puzzletools.client.util.TitledDialogBox;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.ui.FlexTable;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.VerticalPanel;

/**
 * Dialog box that asks for width and height of the new grid.
 */
public class NewGridDialogBox extends TitledDialogBox {
	public static final String TITLE = "New Grid";
	public static final String DESCRIPTION = "Valid values are 1-8 for both width and height.";
	
	private static int MAX_VALUE = 8;
	
	private Label description = new Label(DESCRIPTION);
	private FlexTable flexTbl = new FlexTable();
	private TextBox widthBox = new TextBox();
	private TextBox heightBox = new TextBox();
	private OkCancelPanel okCancel = new OkCancelPanel();
	private VerticalPanel vPanel = new VerticalPanel();
	
	
	public NewGridDialogBox(int width, int height) {
		super(TITLE);
		setAnimationEnabled(true);
		
		description.addStyleName("helpText");
		
		widthBox.setText(Integer.toString(width));
		widthBox.addStyleName("numericBox");
		heightBox.setText(Integer.toString(height));
		heightBox.addStyleName("numericBox");
		
	    // Let's disallow non-numeric entry in the normal text box.
		NumericKeyboardListener numericHandler = new NumericKeyboardListener();
		widthBox.addKeyPressHandler(numericHandler);
		heightBox.addKeyPressHandler(numericHandler);
		
		vPanel.add(description);
		vPanel.add(flexTbl);
		vPanel.add(okCancel);
		vPanel.setCellHorizontalAlignment(okCancel, HasHorizontalAlignment.ALIGN_CENTER);

		flexTbl.setText(1, 0, "Width:");
		flexTbl.setWidget(1, 1, widthBox);
		flexTbl.setText(2, 0, "Height:");
		flexTbl.setWidget(2, 1, heightBox);
		setWidget(vPanel);
	}

	public void addCancelClickHandler(ClickHandler handler) {
		okCancel.addCancelButtonHandler(handler);
	}

	public void addOkClickHandler(ClickHandler handler) {
		okCancel.addOkButtonHandler(handler);
	}

	public static int getValueFromNumericTextBox(TextBox text) {
		int value = 1;
		try {
			value = Integer.parseInt(text.getText());
		} catch (Exception e) {
		}
		
		if (value < 1) {
			value = 1;
		} else if (value >= MAX_VALUE) {
			value = MAX_VALUE;
		}
		return value;
		
	}
	
	public int getWidthAsInt() {
		return getValueFromNumericTextBox(widthBox);
	}
	
	public int getHeightAsInt() {
		return getValueFromNumericTextBox(heightBox);
	}	
}
