package com.analogbit.puzzletools.client.slidingblocks.gui;

import java.util.List;

import com.analogbit.puzzletools.client.slidingblocks.state.Grid;
import com.analogbit.puzzletools.client.slidingblocks.state.SolutionStep;
import com.analogbit.puzzletools.client.util.PreviousNextPanel.DISPLAY_MODE;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.HorizontalPanel;

/**
 * Contains and controls the solution grid.
 */
public class SolutionPanel extends WizardPanel {

	private SolutionGridDisplay solutionGrid;
	private Button stepFwdButton;
	private Button stepBackButton;
	
	public SolutionPanel() {
		super("Solution", "");
		addStyleName("solutionpanel");

		solutionGrid = new SolutionGridDisplay();

		stepBackButton = new Button("Step Back");
		stepBackButton.addStyleName("backButton");
		
		stepFwdButton = new Button("Step Forward");
		stepFwdButton.addStyleName("fwdButton");
		
		HorizontalPanel hp = new HorizontalPanel();
		hp.addStyleName("backFwdPanel");
		hp.add(stepBackButton);
		hp.add(stepFwdButton);
		hp.setCellHorizontalAlignment(stepBackButton, ALIGN_RIGHT);
		hp.setCellHorizontalAlignment(stepFwdButton, ALIGN_LEFT);
	
		this.add(solutionGrid);
		this.add(hp);

//		this.setCellHorizontalAlignment(hp, HasHorizontalAlignment.ALIGN_CENTER);
//		this.setCellHorizontalAlignment(solutionGrid, HasHorizontalAlignment.ALIGN_CENTER);
		
		stepFwdButton.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				solutionGrid.displayNext();
				stepBackButton.setEnabled(true);
				if (!solutionGrid.hasNext()) {
					stepFwdButton.setEnabled(false);
				}
			}
		});
		
		stepBackButton.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				solutionGrid.displayPrevious();
				stepFwdButton.setEnabled(true);
				if (!solutionGrid.hasPrevious()) {
					stepBackButton.setEnabled(false);
				}
			}
		});
		
		setNextDisplayMode(DISPLAY_MODE.HIDDEN);
	}
	
	public void setSolution(Grid originalGrid, List<SolutionStep> solution) {
		super.setDescription("We have successfully computed the answer. It takes " + solution.size() + " moves to reach the goal state.");
		solutionGrid.setSolution(originalGrid, solution);
		stepFwdButton.setEnabled(true);
		stepBackButton.setEnabled(false);
	}
	
}
